import 'dart:html';
import 'dart:async';
import 'package:angular/angular.dart';
import 'package:stream_transform/stream_transform.dart';

@Component(
  selector: 'rp-range',
  templateUrl: 'rp_range.html',
  styleUrls: ['rp_range.css'],
  directives: [],
  pipes: [AsyncPipe]
)
class RangeComponent implements AfterViewInit, OnInit, OnDestroy {

  RangeComponent();

  StreamController _onMove = StreamController<MoveOutput>.broadcast();
  @Output()
  Stream get onMove => _onMove.stream
  .where((data) {
    return data.from % STEP_RANGE == 0 && data.from.toInt() != prevMoveValue;
  })
      .map((data) {
        prevMoveValue = data.from.toInt();
        data.from = ((data.from).toInt() / 100);
        data.to = ((data.to).toInt() / 100);
      print('data.from = ${data.from}');
      print('data.to = ${data.to}');

       _moveFromPosition = data.from;
        _moveToPosition = data.to;

      return data;
    }
  );

  StreamController _onZoom = StreamController<double>();
  @Output()
  Stream get onZoom => _onZoom.stream
      .where((data) {
        return data % STEP_ZOOM == 0 && data.toInt() != prevZoomValue;
  })
    .map((data) {return data.toInt();})
    .map((data) {
      prevZoomValue = data;
      print('prevZoomValue $prevZoomValue');
      print('zoom = ${data / 100}');

      _zoomPosition = data / 100;

      return data / 100;
    }
  );

  @Input('stepZoom')
  double STEP_ZOOM = 1;
  int prevZoomValue = 0;

  @Input('stepMove')
  double STEP_RANGE = 1;
  int prevMoveValue = 0;

  @Input('width')
  int containerWidth = 300;

  @Input('minSliderWidth')
  int SLIDER_MIN_WIDTH = 50;

  @ViewChild('moveBtn') Element moveBtn;
  @ViewChild('moveBtnContainer') Element moveBtnContainer;

  @ViewChild('moveBtnLeft') Element moveBtnLeft;
  @ViewChild('moveBtnRight') Element moveBtnRight;
  
  double _percentSlide;
  double _percentZoom;

  double _moveFromPosition;
  double _moveToPosition;
  double _zoomPosition = 0;

  bool _centerBtnMouseDown = false;
  bool _centerBtnMouseMove = false;
  bool _centerBtnMouseUp = false;
  bool _centerBtnSettedPos = false;

  bool _isLeftSideOverflow = false;
  bool _isRightSideOverflow = false;

  int _centerBtnPossitionStart = 0;
  int _centerBtnPossitionEnd;
  int _centerBtnPossitionDif;

  int _moveSliderWidth;

  bool _rightBtnMouseDown = false;
  bool _rightBtnMouseMove = false;
  bool _rightBtnMouseUp = false;
  int _rightBtnPossitionStart;
  int _rightBtnPossitionStartMove;
  int _rightBtnPossitionEnd;
  int _rightBtnPossitionDif;

  bool _leftBtnMouseDown = false;
  bool _leftBtnMouseMove = false;
  bool _leftBtnMouseUp = false;
  int _leftBtnPossitionStart = 0;
  int _leftBtnPossitionStartMove = 0;
  int _leftBtnPossitionEnd;
  int _leftBtnPossitionDif;
  int _leftCurrentDif = 0;

  @override
  void ngOnInit() {
//    onMove.listen((d) {});
//    onZoom.listen((d) {});
  }

  @override
  void ngAfterViewInit() {
    document.addEventListener("mouseup", _documentMouseUp);
    document.addEventListener("mousemove", _documentMouseMove);

    if (containerWidth == 0) {
      containerWidth = moveBtnContainer.clientWidth;
    } else {
      moveBtnContainer.style.width = '${containerWidth}px';
    }

    _moveSliderWidth = SLIDER_MIN_WIDTH;

    _rightBtnPossitionEnd = containerWidth - _moveSliderWidth;
    _leftBtnPossitionStart = _getCurrentLeftPosition();

    moveBtn.style.right = '${containerWidth - SLIDER_MIN_WIDTH}px';

    _percentZoom = (containerWidth - SLIDER_MIN_WIDTH) / 100;
    _percentSlide = (containerWidth) / 100;

    _onZoom.add(0);
    _onMove.add(_getPositionPercents());
  }

  _documentMouseMove(e) {
    if (_centerBtnMouseDown) {
      _slide(e);
      MoveOutput pos = _getPositionPercents();

      if (pos.from >= 0 && pos.to >= 0 && pos.from <= 100 - (SLIDER_MIN_WIDTH /_percentSlide) && pos.to <= 100) {
        _onMove.add(pos);
      }
    }

    if (_rightBtnMouseDown) {
      _handleRightMouseMove(e);
    }

    if (_leftBtnMouseDown) {
      _handleLeftMouseMove(e);
    }

    if (_rightBtnMouseDown || _leftBtnMouseDown) {
      double zoom = _getZoom();

      if (zoom >= 0 && zoom <= 100) {
        _onZoom.add(zoom);
      }
    }

  }

  _documentMouseUp(e) {
    MoveOutput pos = _getPositionPercents();

    if (_leftBtnMouseDown || _rightBtnMouseDown || _centerBtnMouseDown) {
      if (pos.from >= 0 && pos.to >= 0 && pos.from <= 100 - (SLIDER_MIN_WIDTH / _percentSlide) && pos.to <= 100) {
        _onMove.add(pos);
      }
    }


    if (_centerBtnMouseMove) {
      _centerBtnMouseDown = false;
      _centerBtnMouseMove = false;
      _centerBtnMouseUp = true;
      _savePosition();
    }

    _rightBtnMouseDown = false;
    _rightBtnMouseMove = false;
    _rightBtnMouseUp = true;

    _leftBtnMouseUp = true;
    _leftBtnMouseDown = false;
    _leftBtnMouseMove = false;

    _moveSliderWidth = moveBtn.clientWidth;
    _leftBtnPossitionStart = _getCurrentLeftPosition();
  }

  handleCenterMouseDown(e) {
    _centerBtnMouseDown = true;
    _centerBtnMouseUp = false;
  }

  _handleCenterMouseMove(MouseEvent e) {
    _centerBtnSettedPos = false;
  }

  _handleCenterMouseUp(e) {
    _savePosition();
  }

  _slide(MouseEvent e) {
    _centerBtnSettedPos = false;

    if (!_centerBtnMouseMove && _centerBtnMouseDown && !_centerBtnMouseUp) {
      _centerBtnPossitionStart = e.client.x;
    }


    if (_centerBtnMouseDown) {
      _centerBtnMouseMove = true;
      _centerBtnPossitionDif = e.client.x - _centerBtnPossitionStart;

      if (moveBtn.offsetLeft + _centerBtnPossitionDif >= 0) {
        _isLeftSideOverflow = false;
      } else {
        _isLeftSideOverflow = true;
        _setLeftPoss(0);
        _setRightPoss(containerWidth - _moveSliderWidth);
      }

      if (moveBtn.offsetLeft + moveBtn.clientWidth + _centerBtnPossitionDif <= containerWidth) {
        _isRightSideOverflow = false;
      } else {
        _isRightSideOverflow = true;
        _setLeftPoss(containerWidth - _moveSliderWidth);
        _setRightPoss(0);
      }

      if (!_isLeftSideOverflow && !_isRightSideOverflow) {
        this.moveBtn.style.transform = 'translateX(${_centerBtnPossitionDif}px)';
      } else {
        this.moveBtn.style.transform = 'translateX(0px)';
      }
    }
  }

  _mouseBtnUp() {
    _centerBtnMouseUp = true;
    _centerBtnMouseDown = false;
    _centerBtnMouseMove = false;
  }

  _savePosition() {
    _mouseBtnUp();

    if (!_centerBtnSettedPos && !_isLeftSideOverflow && !_isRightSideOverflow) {
      this.moveBtn.style.transform = '';
      int left = this.moveBtn.style.left != '' ? int.parse(this.moveBtn.style.left.replaceAll('px', '')) : 0;
      _setLeftPoss(left + _centerBtnPossitionDif);
      _leftBtnPossitionEnd = left;
      _setRightPoss(_getCurrentRightPosition());

      _centerBtnSettedPos = true;
    }
  }

  handleLeftMouseDown(MouseEvent e) {
    _leftBtnMouseDown = true;
    _leftBtnMouseUp = false;
  }

  _handleLeftMouseMove(MouseEvent e) {
    if (!_leftBtnMouseMove && _leftBtnMouseDown && !_leftBtnMouseUp) {
      _leftBtnPossitionStartMove = e.client.x;
      _leftBtnPossitionStart = _getCurrentLeftPosition();
      _rightBtnPossitionStartMove = _getCurrentRightPosition();
    }



    if (_leftBtnMouseDown) {
      _leftBtnMouseMove = true;

      _leftBtnPossitionDif = e.client.x - _leftBtnPossitionStartMove;
      if (_leftCurrentDif != _leftBtnPossitionDif) {
        _leftCurrentDif = _leftBtnPossitionDif;
      } else {
        return;
      }


      if (moveBtn.clientWidth >= SLIDER_MIN_WIDTH && _leftBtnPossitionDif != 0) {
        int left = _getCurrentLeftPosition();

        if (_leftBtnPossitionStart + _leftBtnPossitionDif >= 0) {

          if (_leftBtnPossitionDif > 0 && containerWidth - left - _rightBtnPossitionStartMove - SLIDER_MIN_WIDTH <= 0) {
            _setLeftPoss(containerWidth - _rightBtnPossitionStartMove - SLIDER_MIN_WIDTH);
            _setRightPoss(_rightBtnPossitionStartMove);
            return;
          }

          _setRightPoss(_rightBtnPossitionStartMove);
          _setLeftPoss(_leftBtnPossitionStart + _leftBtnPossitionDif);
        } else {
          _setLeftPoss(0);
        }

      }
    }
  }

  handleLeftMouseUp(MouseEvent e) {
    _rightBtnMouseUp = true;
    _rightBtnMouseDown = false;
    _rightBtnMouseMove = false;
  }

  handleRightMouseDown(MouseEvent e) {
    _rightBtnMouseDown = true;
    _rightBtnMouseUp = false;
  }

  _handleRightMouseMove(MouseEvent e) {

    if (!_rightBtnMouseMove && _rightBtnMouseDown && !_rightBtnMouseUp) {
      _rightBtnPossitionStartMove = e.client.x;
    }

    if (_rightBtnMouseDown) {
      _rightBtnMouseMove = true;
      _rightBtnPossitionDif = e.client.x - _rightBtnPossitionStartMove;

      if (moveBtn.clientWidth >= SLIDER_MIN_WIDTH) {
        int left = _getCurrentLeftPosition();
        _setLeftPoss(left);

        _rightBtnPossitionEnd = containerWidth - _moveSliderWidth - left;
        if (_rightBtnPossitionDif + _leftBtnPossitionStart + _moveSliderWidth <= containerWidth) {
          _setRightPoss(_rightBtnPossitionEnd - _rightBtnPossitionDif);
        } else {
          _setRightPoss(0);
        }

      }

    }

  }

  int _getCurrentLeftPosition() {
    return this.moveBtn.style.left != '' ? int.parse(this.moveBtn.style.left.replaceAll('px', '')) : 0;
  }

  int _getCurrentRightPosition() {
    return this.moveBtn.style.left != '' ? containerWidth - (int.parse(this.moveBtn.style.left.replaceAll('px', '')) + _moveSliderWidth) : containerWidth - SLIDER_MIN_WIDTH;
  }

  _setRightPoss(right) {
    if (_moveSliderWidth >= SLIDER_MIN_WIDTH) {
      _rightBtnPossitionEnd = right;
      moveBtn.style.right = '${_rightBtnPossitionEnd}px';
    }
  }

  _setLeftPoss(left) {
    if (_moveSliderWidth >= SLIDER_MIN_WIDTH) {
      moveBtn.style.left = '${left}px';
    }
  }

  MoveOutput _getPositionPercents() {
    int offset = _centerBtnPossitionDif != null ? _centerBtnPossitionDif : 0;
    double from = (_getCurrentLeftPosition() + offset) /_percentSlide;

    int sliderWidth = (containerWidth) - int.parse(this.moveBtn.style.right.replaceAll('px', ''))
        -  _getCurrentLeftPosition();

    double to = (_getCurrentLeftPosition() + sliderWidth).toDouble();
    to = (to + offset) /_percentSlide;

    return new MoveOutput()
      ..from = from > 0 ? from : 0
      ..to = to > 1 ? to : 1;
  }

  double _getZoom() {
    int sliderWidth = containerWidth - SLIDER_MIN_WIDTH - int.parse(this.moveBtn.style.right.replaceAll('px', ''))
        -  _getCurrentLeftPosition();

    double zoom = ((sliderWidth) /_percentZoom);
    return zoom;
  }

  @override
  void dispose() {
    _onMove.close();
    _onZoom.close();
    document.removeEventListener('mousemove', _documentMouseMove);
    document.removeEventListener('mouseup', _documentMouseUp);
  }

  @override
  void ngOnDestroy() {

  }

}

class MoveOutput {
  double from;
  double to;
}